#ifndef __OGG_VORBIS_H__
#define __OGG_VORBIS_H__


#include <vorbis/codec.h>
#include <vorbis/vorbisfile.h>
#include <string.h>

#include "InputPlugin.h"



// derive a class from CInputPlugin
// this class will be used to get information on what files
// does it support, information about the files and 
// configuration etc.

class COggVorbisPlugin : public CInputPlugin
{
public:
	COggVorbisPlugin(HINSTANCE hDLLInstance,HWND hApolloWnd) : _hDLLInstance(hDLLInstance), _hApolloWnd(hApolloWnd) {}
	char *__cdecl GetDescription();

	CInputDecoder *__cdecl Open(char *filename,int audioDataOffset);
	BOOL __cdecl GetInfo(char *filename,int audioDataOffset,TrackInfo *trackInfo);
	void __cdecl AdditionalInfo(char *filename,int audioDataOffset);
	BOOL __cdecl IsSuitableFile(char *filename,int audioDataOffset);
	char *__cdecl GetExtensions();

	void __cdecl Config();
	void __cdecl About();

	void __cdecl Close() { delete this; }

private:

	HINSTANCE _hDLLInstance;
	HWND _hApolloWnd;
};


//typedef struct {
//  size_t (*read_func)  (void *ptr, size_t size, size_t nmemb, void *datasource);
//  int	(*seek_func)  (void *datasource, int64_t offset, int whence);
//  int	(*close_func) (void *datasource);
//  long   (*tell_func)  (void *datasource);
//} ov_callbacks;


size_t vfcb_i_read (void *ptr, size_t size, size_t nmemb, void *datasource);
int vfcb_i_seek(void *datasource, ogg_int64_t offset, int whence);
long vfcb_i_tell(void *datasource);
int vfcb_i_close(void *datasource);



// derive a class from CInputDecoder
// this class will do the actual decoding

class COggVorbisDecoder : public CInputDecoder
{
public:
	COggVorbisDecoder(char *filename,int audioDataOffset);

	int __cdecl GetNextAudioChunk(void **buffer);
	int __cdecl SetPosition(int newPosition);
	
	void __cdecl SetEqualizer(BOOL equalize);
	void __cdecl AdjustEqualizer(int equalizerValues[16]);

	void __cdecl Close() { delete this; }

private:
	~COggVorbisDecoder();

//friend size_t vfcb_read (void *ptr, size_t size, size_t nmemb, void *datasource);
//friend int vfcb_seek(void *datasource, ogg_int64_t offset, int whence);
//friend long vfcb_tell(void *datasource);
//friend int vfcb_close(void *datasource);


	HANDLE _hFile;
	char *_buffer;
	int _playingTime;
	int _beginOfRAWData;
	OggVorbis_File _vf;
	ov_callbacks _cb;
        int _v_current_section;
};

//ov_callbacks __cb = { read_func: vfcb_read, seek_func: vfcb_seek, tell_func: vfcb_tell, close_func: vfcb_close };


#endif//__OGG_VORBIS_H__
